@@ -233,6 +233,12 @@ class DObject(AbstractDElement):
 			return sorted(list(classes), key = lambda cls: cls.order)
 		return list(classes)
 	
+	def get_class_names(self, ordered = False, superclasses = False):
+		# returns [name, ...]
+		
+		return [cls.name for cls in self.get_classes(ordered, superclasses)]
+		
+	
 	def has_class(self, name = None):
 		
 		if name.__class__.__name__ == "DClass":
